@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1999-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -25,13 +25,17 @@
 
 #include "os-interface.h"
 #include "nv-linux.h"
+#include <linux/iommu.h>
 #include "nv-caps-imex.h"
 
+#include "nv-platform.h"
+
 #include "nv-time.h"
 
 #include <linux/mmzone.h>
 #include <linux/numa.h>
 #include <linux/cpuset.h>
+#include <linux/sys_soc.h>
 
 extern char *NVreg_TemporaryFilePath;
 
@@ -43,13 +47,16 @@ extern nv_linux_state_t nv_ctl_device;
 
 extern nv_kthread_q_t nv_kthread_q;
 
-NvU32 os_page_size  = PAGE_SIZE;
-NvU64 os_page_mask  = NV_PAGE_MASK;
-NvU8  os_page_shift = PAGE_SHIFT;
+NvU64 os_page_size     = PAGE_SIZE;
+NvU64 os_max_page_size = PAGE_SIZE << NV_MAX_PAGE_ORDER;
+NvU64 os_page_mask     = NV_PAGE_MASK;
+NvU8  os_page_shift    = PAGE_SHIFT;
+
 NvBool os_cc_enabled = 0;
 NvBool os_cc_sev_snp_enabled = 0;
 NvBool os_cc_snp_vtom_enabled = 0;
 NvBool os_cc_tdx_enabled = 0;
+NvBool os_cc_sme_enabled = 0;
 
 NvBool os_dma_buf_enabled = NV_FALSE;
 
@@ -441,11 +448,6 @@ void *NV_API_CALL os_mem_copy(
      * When performing memcpy for memory mapped as device, memcpy_[to/from]io
      * must be used. WAR to check the source and destination to determine the
      * correct memcpy_io to use.
-     *
-     * This WAR is limited to just aarch64 for now because the address range used
-     * to map ioremap and vmalloc is different on ppc64le, and is_vmalloc_addr()
-     * does not correctly handle this. is_ioremap_addr() is needed instead. This
-     * will have to be addressed when reorganizing RM to use the new memset model.
      */
     if (is_vmalloc_addr(dst) && !is_vmalloc_addr(src))
     {
@@ -515,11 +517,6 @@ void* NV_API_CALL os_mem_set(
      *
      * WAR to check the destination to determine if the memory is of type Device
      * or Normal, and use the correct memset.
-     *
-     * This WAR is limited to just aarch64 for now because the address range used
-     * to map ioremap and vmalloc is different on ppc64le, and is_vmalloc_addr()
-     * does not correctly handle this. is_ioremap_addr() is needed instead. This
-     * will have to be addressed when reorganizing RM to use the new memset model.
      */
     if (is_vmalloc_addr(dst))
     {
@@ -643,11 +640,11 @@ void NV_API_CALL os_free_mem(void *address)
 
 /*****************************************************************************
 *
-*   Name: osGetCurrentTime
+*   Name: osGetSystemTime
 *
 *****************************************************************************/
 
-NV_STATUS NV_API_CALL os_get_current_time(
+NV_STATUS NV_API_CALL os_get_system_time(
     NvU32 *seconds,
     NvU32 *useconds
 )
@@ -665,16 +662,14 @@ NV_STATUS NV_API_CALL os_get_current_time(
 //
 // Get the High resolution tick count of the system uptime
 //
-NvU64 NV_API_CALL os_get_current_tick_hr(void)
+NvU64 NV_API_CALL os_get_monotonic_time_ns_hr(void)
 {
     struct timespec64 tm;
     ktime_get_raw_ts64(&tm);
     return (NvU64) timespec64_to_ns(&tm);
 }
 
-#if BITS_PER_LONG >= 64
-
-NvU64 NV_API_CALL os_get_current_tick(void)
+NvU64 NV_API_CALL os_get_monotonic_time_ns(void)
 {
 #if defined(NV_JIFFIES_TO_TIMESPEC_PRESENT)
     struct timespec ts;
@@ -687,47 +682,13 @@ NvU64 NV_API_CALL os_get_current_tick(void)
 #endif
 }
 
-NvU64 NV_API_CALL os_get_tick_resolution(void)
+NvU64 NV_API_CALL os_get_monotonic_tick_resolution_ns(void)
 {
     return (NvU64)jiffies_to_usecs(1) * NSEC_PER_USEC;
 }
 
-#else
-
-NvU64 NV_API_CALL os_get_current_tick(void)
-{
-    /*
-     * 'jiffies' overflows regularly on 32-bit builds (unsigned long is 4 bytes
-     * instead of 8 bytes), so it's unwise to build a tick counter on it, since
-     * the rest of the Resman assumes the 'tick' returned from this function is
-     * monotonically increasing and never overflows.
-     *
-     * Instead, use the previous implementation that we've lived with since the
-     * beginning, which uses system clock time to calculate the tick. This is
-     * subject to problems if the system clock time changes dramatically
-     * (more than a second or so) while the Resman is actively tracking a
-     * timeout.
-     */
-    NvU32 seconds, useconds;
-
-    (void) os_get_current_time(&seconds, &useconds);
-
-    return ((NvU64)seconds * NSEC_PER_SEC +
-                 (NvU64)useconds * NSEC_PER_USEC);
-}
-
-NvU64 NV_API_CALL os_get_tick_resolution(void)
-{
-    /*
-     * os_get_current_tick() uses os_get_current_time(), which has
-     * microsecond resolution.
-     */
-    return 1000ULL;
-}
-
-#endif
-
 //---------------------------------------------------------------------------
+
 //
 //  Misc services.
 //
@@ -773,6 +734,18 @@ void NV_API_CALL os_get_current_process_name(char *buf, NvU32 len)
     task_unlock(current);
 }
 
+NV_STATUS NV_API_CALL os_iommu_sva_bind(void *arg, void **handle, NvU32 *pasid)
+{
+    nv_state_t *nv = arg;
+    NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "IOMMU SVA bind failed\n");
+
+    return NV_ERR_INVALID_STATE;
+}
+
+void NV_API_CALL os_iommu_sva_unbind(void *handle)
+{
+}
+
 NV_STATUS NV_API_CALL os_get_current_thread(NvU64 *threadId)
 {
     if (in_interrupt())
@@ -1054,8 +1027,6 @@ void NV_API_CALL os_flush_cpu_write_combine_buffer(void)
 {
 #if defined(NVCPU_X86_64)
     asm volatile("sfence" ::: "memory");
-#elif defined(NVCPU_PPC64LE)
-    __asm__ __volatile__ ("sync" : : : "memory");
 #elif defined(NVCPU_AARCH64)
     asm volatile("dsb st" : : : "memory");
 #else
@@ -1214,8 +1185,6 @@ void NV_API_CALL os_dbg_breakpoint(void)
     __asm__ __volatile__ (".word %c0" :: "i" (KGDB_COMPILED_BREAK));
   #elif defined(NVCPU_AARCH64)
     # warning "Need to implement os_dbg_breakpoint() for aarch64"
-  #elif defined(NVCPU_PPC64LE)
-    __asm__ __volatile__ ("trap");
   #endif // NVCPU_*
 #elif defined(CONFIG_KDB)
     KDB_ENTER();
@@ -1482,7 +1451,7 @@ static NV_STATUS os_get_smbios_header_legacy(NvU64 *pSmbsAddr)
 }
 
 // This function is needed only if "efi" is enabled.
-#if (defined(NV_LINUX_EFI_H_PRESENT) && defined(CONFIG_EFI))
+#if defined(CONFIG_EFI)
 static NV_STATUS os_verify_smbios_header_uefi(NvU64 smbsAddr)
 {
     NV_STATUS status = NV_ERR_OBJECT_NOT_FOUND;
@@ -1525,8 +1494,7 @@ static NV_STATUS os_get_smbios_header_uefi(NvU64 *pSmbsAddr)
 {
     NV_STATUS status = NV_ERR_OPERATING_SYSTEM;
 
-// Make sure that efi.h is present before using "struct efi".
-#if (defined(NV_LINUX_EFI_H_PRESENT) && defined(CONFIG_EFI))
+#if defined(CONFIG_EFI)
 
 // Make sure that efi.h has SMBIOS3_TABLE_GUID present.
 #if defined(SMBIOS3_TABLE_GUID)
@@ -1593,9 +1561,7 @@ NV_STATUS NV_API_CALL os_get_acpi_rsdp_from_uefi
 
     *pRsdpAddr = 0;
 
-// Make sure that efi.h is present before using "struct efi".
-#if (defined(NV_LINUX_EFI_H_PRESENT) && defined(CONFIG_EFI))
-
+#if defined(CONFIG_EFI)
     if (efi.acpi20 != EFI_INVALID_TABLE_ADDR)
     {
         *pRsdpAddr = efi.acpi20;
@@ -1642,7 +1608,7 @@ NV_STATUS NV_API_CALL os_alloc_pages_node
     NV_STATUS status = NV_ERR_NOT_SUPPORTED;
 
 #if defined(__GFP_THISNODE) && defined(GFP_HIGHUSER_MOVABLE) && \
-    defined(__GFP_COMP) && defined(__GFP_NORETRY) && defined(__GFP_NOWARN)
+    defined(__GFP_COMP) && defined(__GFP_NOWARN)
     gfp_t gfp_mask;
     struct page *alloc_addr;
     unsigned int order = get_order(size);
@@ -1659,13 +1625,11 @@ NV_STATUS NV_API_CALL os_alloc_pages_node
      *                              pages, which is needed in order to use
      *                              vm_insert_page API.
      *
-     * 4. __GFP_NORETRY:            Used to avoid the Linux kernel OOM killer.
-     *
-     * 5. __GFP_NOWARN:             Used to avoid a WARN_ON in the slowpath if
+     * 4. __GFP_NOWARN:             Used to avoid a WARN_ON in the slowpath if
      *                              the requested order is too large (just fail
      *                              instead).
      *
-     * 6. (Optional) __GFP_RECLAIM: Used to allow/forbid reclaim.
+     * 5. (Optional) __GFP_RECLAIM: Used to allow/forbid reclaim.
      *                              This is part of GFP_USER and consequently
      *                              GFP_HIGHUSER_MOVABLE.
      *
@@ -1679,7 +1643,30 @@ NV_STATUS NV_API_CALL os_alloc_pages_node
      */
 
     gfp_mask = __GFP_THISNODE | GFP_HIGHUSER_MOVABLE | __GFP_COMP |
-               __GFP_NORETRY | __GFP_NOWARN;
+               __GFP_NOWARN;
+    
+#if defined(__GFP_RETRY_MAYFAIL)
+
+    /*
+     * __GFP_RETRY_MAYFAIL :  Used to avoid the Linux kernel OOM killer.
+     *                        To help PMA on paths where UVM might be
+     *                        in memory over subscription. This gives UVM 
+     *                        a chance to free memory before invoking any 
+     *                        action from the OOM killer.
+     *                        Freeing non-essential memory will also benefit 
+     *                        the system as a whole.
+     */
+
+    gfp_mask |= __GFP_RETRY_MAYFAIL;
+#elif defined(__GFP_NORETRY)
+
+    /*
+     *  __GFP_NORETRY :       Use __GFP_NORETRY on older kernels where
+     *                        __GFP_RETRY_MAYFAIL is not present.
+     */
+
+    gfp_mask |= __GFP_NORETRY;
+#endif
 
 #if defined(__GFP_RECLAIM)
     if (flag & NV_ALLOC_PAGES_NODE_SKIP_RECLAIM)
@@ -1856,11 +1843,7 @@ NV_STATUS NV_API_CALL os_write_file
     int num_retries = NV_MAX_NUM_FILE_IO_RETRIES;
 
 retry:
-#if defined(NV_KERNEL_WRITE_HAS_POINTER_POS_ARG)
     num_written = kernel_write(pFile, pBuffer, size, &f_pos);
-#else
-    num_written = kernel_write(pFile, pBuffer, size, f_pos);
-#endif
     if (num_written < 0)
     {
         return NV_ERR_OPERATING_SYSTEM;
@@ -1900,11 +1883,7 @@ NV_STATUS NV_API_CALL os_read_file
     int num_retries = NV_MAX_NUM_FILE_IO_RETRIES;
 
 retry:
-#if defined(NV_KERNEL_READ_HAS_POINTER_POS_ARG)
     num_read = kernel_read(pFile, pBuffer, size, &f_pos);
-#else
-    num_read = kernel_read(pFile, f_pos, pBuffer, size);
-#endif
     if (num_read < 0)
     {
         return NV_ERR_OPERATING_SYSTEM;
@@ -2009,15 +1988,29 @@ NV_STATUS NV_API_CALL os_get_random_bytes
     NvU16 numBytes
 )
 {
-#if defined NV_WAIT_FOR_RANDOM_BYTES_PRESENT
     if (wait_for_random_bytes() < 0)
         return NV_ERR_NOT_READY;
-#endif
 
     get_random_bytes(bytes, numBytes);
     return NV_OK;
 }
 
+NvU32 NV_API_CALL os_get_current_process_flags
+(
+    void
+)
+{
+    NvU32 flags = OS_CURRENT_PROCESS_FLAG_NONE;
+
+    if (current->flags & PF_EXITING)
+        flags |= OS_CURRENT_PROCESS_FLAG_EXITING;
+
+    if (current->flags & PF_KTHREAD)
+        flags |= OS_CURRENT_PROCESS_FLAG_KERNEL_THREAD;
+
+    return flags;
+}
+
 NV_STATUS NV_API_CALL os_alloc_wait_queue
 (
     os_wait_queue **wq
@@ -2064,6 +2057,59 @@ void NV_API_CALL os_wake_up
     complete_all(&wq->q);
 }
 
+static bool os_platform_is_fpga(void)
+{
+    const struct soc_device_attribute soc_attrs[] = {
+        { .revision = "*FPGA" },
+        {/* sentinel */}
+    };
+
+    if (soc_device_match(soc_attrs)) {
+        return true;
+    }
+
+    return false;
+}
+
+static bool os_platform_is_vdk(void)
+{
+    const struct soc_device_attribute soc_attrs[] = {
+        { .revision = "VDK" },
+        {/* sentinel */}
+    };
+
+    if (soc_device_match(soc_attrs)) {
+        return true;
+    }
+
+    return false;
+}
+
+    NV_STATUS NV_API_CALL os_get_tegra_platform
+(
+    NvU32 *mode
+)
+{
+#if NV_SUPPORTS_PLATFORM_DISPLAY_DEVICE
+    if (os_platform_is_fpga())
+    {
+        *mode = NV_OS_TEGRA_PLATFORM_FPGA;
+    }
+    else if (os_platform_is_vdk())
+    {
+        *mode = NV_OS_TEGRA_PLATFORM_SIM;
+    }
+    else
+    {
+        *mode = NV_OS_TEGRA_PLATFORM_SILICON;
+    }
+
+    return NV_OK;
+#else
+    return NV_ERR_NOT_SUPPORTED;
+#endif
+}
+
 nv_cap_t* NV_API_CALL os_nv_cap_init
 (
     const char *path
@@ -2217,7 +2263,7 @@ static int os_numa_verify_gpu_memory_zone(struct notifier_block *nb,
     return NOTIFY_OK;
 }
 
-#define ADD_REMOVE_GPU_MEMORY_NUM_SEGMENTS 4
+#define ADD_REMOVE_GPU_MEMORY_NUM_SEGMENTS 128
 
 NV_STATUS NV_API_CALL os_numa_add_gpu_memory
 (
@@ -2334,3 +2380,16 @@ NV_STATUS NV_API_CALL os_find_ns_pid(void *pid_info, NvU32 *ns_pid)
     return NV_ERR_NOT_SUPPORTED;
 }
 
+NvBool NV_API_CALL os_is_init_ns(void)
+{
+    return NV_TRUE;
+}
+
+NV_STATUS NV_API_CALL os_device_vm_present(void)
+{
+#if defined(NV_DEVICE_VM_BUILD)
+    return NV_OK;
+#else
+    return NV_ERR_NOT_SUPPORTED;
+#endif
+}
